//Calendar implementation for the HTWK Leipzig timetable. Evaluation and display of the individual dates in iCal format.
//Copyright (C) 2024 HTWKalender support@htwkalender.de

//This program is free software: you can redistribute it and/or modify
//it under the terms of the GNU Affero General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.

//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU Affero General Public License for more details.

//You should have received a copy of the GNU Affero General Public License
//along with this program.  If not, see <https://www.gnu.org/licenses/>.

package ical

import (
	"bytes"
	"encoding/json"
	"htwkalender/model"
	"htwkalender/service/db"
	"htwkalender/service/feed"
	"time"

	"github.com/jordic/goics"
	"github.com/pocketbase/pocketbase"
	"github.com/pocketbase/pocketbase/apis"
)

const expirationTime = 5 * time.Minute

func Feed(app *pocketbase.PocketBase, token string) (string, error) {
	var result string

	icalFeed, err := db.FindFeedByToken(token, app)
	if icalFeed == nil && err != nil {
		return "", err
	}

	modules := make(map[string]model.FeedCollection)
	var modulesArray []model.FeedCollection
	_ = json.Unmarshal([]byte(icalFeed.Modules), &modulesArray)

	for _, module := range modulesArray {
		modules[module.UUID] = module
	}

	newFeed, createFeedErr := createFeedForToken(app, modules)
	if createFeedErr != nil {
		return "", createFeedErr
	}
	result = newFeed.Content

	return result, nil
}

func createFeedForToken(app *pocketbase.PocketBase, modules map[string]model.FeedCollection) (*model.FeedModel, error) {
	events, err := db.GetPlanForModules(app, modules)
	if err != nil {
		return nil, apis.NewNotFoundError("Could not fetch events", err)
	}

	// Combine Events
	events = feed.CombineEventsInFeed(events)

	b := bytes.Buffer{}
	goics.NewICalEncode(&b).Encode(IcalModel{Events: events, Mapping: modules})
	icalFeed := &model.FeedModel{Content: b.String(), ExpiresAt: time.Now().Add(expirationTime)}
	return icalFeed, nil
}

func CreateIndividualFeed(requestBody []byte, app *pocketbase.PocketBase) (string, error) {
	var modules []model.FeedCollection

	err := json.Unmarshal(requestBody, &modules)
	if err != nil {
		return "", apis.NewNotFoundError("Could not parse request body", err)
	}

	var icalFeed model.Feed
	jsonModules, _ := json.Marshal(modules)
	icalFeed.Modules = string(jsonModules)

	collection, dbError := db.FindCollection(app, "feeds")
	if dbError != nil {
		return "", apis.NewNotFoundError("Collection could not be found", dbError)
	}

	record, err := db.SaveFeed(icalFeed, collection, app)
	if err != nil {
		return "", apis.NewNotFoundError("Could not save feed", err)
	}

	return record.Id, nil
}
